/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.linux.usermanager.info;

import com.smardec.demo.j2native.linux.usermanager.GroupManager;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;


public class UIUserPropertiesDialog extends JDialog {
    private UserInfo userInfo;

    private JTextField userNameField;
    private JTextField fullNameField;
    private JPasswordField passwordField;
    private JPasswordField confPasswordField;
    private JTextField homeDirField;
    private JTextField loginShellField;

    private JCheckBox accountExpBox;
    private JCheckBox accountLockedBox;
    private JLabel jLabelAccountExpired;
    private JTextField yearExpField;
    private JTextField monthExpField;
    private JTextField dayExpField;

    private JLabel lastPassChangeLabel;
    private JCheckBox passExpBox;
    private JLabel jLabelDaysBeforeAllowed;
    private JLabel jLabelDaysBeforeRequired;
    private JLabel jLabelDaysWarnBeforeChange;
    private JLabel jLabelDaysBeforeInactive;
    private JTextField daysBeforeAllowedField;
    private JTextField daysBeforeRequiredField;
    private JTextField daysWarnBeforeChField;
    private JTextField daysBeforeAccInactiveField;

    private JList groupsSelectionList;
    private JComboBox primaryGroupBox;

    private boolean root;

    public UIUserPropertiesDialog(Frame parentFrame, UserInfo userInfo, boolean root) {
        super(parentFrame, "User Properties: <" + userInfo.getIdent() + ">", true);

        this.root = root;
        this.userInfo = userInfo;

        Container cp = this.getContentPane();
        cp.setLayout(new BorderLayout());

        JPanel mainWindowPanel = new JPanel(new GridBagLayout(), true);

        JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
        JPanel userDataTab = createUserDataTab();
        JPanel accountInfoTab = createAccountInfoTab();
        JPanel passInfoTab = createPasswordInfoTab();
        JPanel groupsTab = createGroupsTab();

        tabbedPane.addTab("User Data", new JScrollPane(userDataTab));
        tabbedPane.addTab("Account Info", new JScrollPane(accountInfoTab));
        tabbedPane.addTab("Password Info", new JScrollPane(passInfoTab));
        tabbedPane.addTab("Groups", groupsTab);
        constrain(mainWindowPanel, tabbedPane, 0, 0, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);

        JButton okBtn = new JButton("Ok");
        okBtn.setEnabled(isRoot());
        okBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("JButton: \"Ok\"");
            }
        });
//    applyFilterBtn.setSize(applyFilterBtn.getWidth(), filterField.getHeight());
        constrain(mainWindowPanel, okBtn, 0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 1.0, 0.0, 5, 5, 5, 5);

        JButton cancelBtn = new JButton("Cancel");
        cancelBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                UIUserPropertiesDialog.this.dispose();
            }
        });
//    applyFilterBtn.setSize(applyFilterBtn.getWidth(), filterField.getHeight());
        constrain(mainWindowPanel, cancelBtn, 1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        cp.add(mainWindowPanel, BorderLayout.CENTER);
        pack();

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int frameWidth = 410;
        int frameHeight = 300;
        setBounds((screenSize.width - frameWidth) / 2, (screenSize.height - frameHeight) / 2, frameWidth, frameHeight);
        // init values
        if (userInfo == null) {
            return;
        }
        userNameField.setText(userInfo.getUserName());
        fullNameField.setText(userInfo.getRealName());
        passwordField.setText("********");
        confPasswordField.setText("********");
        homeDirField.setText(userInfo.getHomeDir());
        loginShellField.setText(userInfo.getShellPrg());

        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTimeInMillis((long) userInfo.getDaysExpire() * 24 * 3600 * 1000);

        if (userInfo.isAccountExpire()) {
            accountExpBox.setSelected(true);
            yearExpField.setText(String.valueOf(gregorianCalendar.get(Calendar.YEAR)));
            monthExpField.setText(String.valueOf(gregorianCalendar.get(Calendar.MONTH) + 1));
            dayExpField.setText(String.valueOf(gregorianCalendar.get(Calendar.DAY_OF_MONTH)));
        } else {
            accountExpBox.setSelected(false);
        }

        accountLockedBox.setSelected(userInfo.isAccountLocked());

        DateFormat dateFormat = new SimpleDateFormat("EEE MMM d yyyy");
        lastPassChangeLabel.setText(dateFormat.format(new Date((long) userInfo.getDateOfLastChange() * 24 * 3600 * 1000)));

        if (userInfo.isPassExpEnabled()) {
            passExpBox.setSelected(true);
            daysBeforeAllowedField.setText((userInfo.getMinDaysChange() < 0) ? "0" : String.valueOf(userInfo.getMinDaysChange()));
            daysBeforeRequiredField.setText(((userInfo.getMaxDaysChange() < 0) || (userInfo.getMaxDaysChange() == UserInfo.MAX_DAYS_BEFORE_CHANGE)) ? "0" : String.valueOf(userInfo.getMaxDaysChange()));
            daysWarnBeforeChField.setText((userInfo.getDaysToWarn() < 0) ? "0" : String.valueOf(userInfo.getDaysToWarn()));
            daysBeforeAccInactiveField.setText((userInfo.getDaysInactive() < 0) ? "0" : String.valueOf(userInfo.getDaysInactive()));
        } else {
            passExpBox.setSelected(false);
            daysBeforeAllowedField.setText("0");
            daysBeforeRequiredField.setText("0");
            daysWarnBeforeChField.setText("0");
            daysBeforeAccInactiveField.setText("0");
        }

        primaryGroupBox.removeAllItems();
        DefaultListModel groupList = new DefaultListModel();
        GroupInfo[] groupInfos = GroupManager.getInstance().getGroupInfos();
        Arrays.sort(groupInfos, new Comparator() {
            public int compare(Object o1, Object o2) {
                GroupInfo group1 = (GroupInfo) o1;
                GroupInfo group2 = (GroupInfo) o2;
                return group1.getGroupName().compareTo(group2.getGroupName());
            }
        });
        java.util.List userGroupsList = userInfo.getGroups();
        for (int i = 0; i < groupInfos.length; i++) {
            GroupInfo grupInfo = groupInfos[i];
            boolean selected = false;
            for (int j = userGroupsList.size() - 1; j >= 0; j--) {
                if (grupInfo.equals(userGroupsList.get(j))) {
                    selected = true;
                    break;
                }
            }
            JCheckBox checkBox = new JCheckBox(grupInfo.getGroupName());
            checkBox.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    JCheckBox groupBox = (JCheckBox) e.getSource();
                    processGroupStateChange(groupBox.getText(), groupBox.isSelected());
                }
            });
            checkBox.setSelected(selected);
            groupList.addElement(checkBox);
            if (userInfo.getGroupId() == grupInfo.getId()) {
                primaryGroupBox.setSelectedItem(grupInfo.getGroupName());
            }
        }
        groupsSelectionList.setModel(groupList);

        tuneControls(null);
    }

    public UserInfo getUserInfo() {
        return userInfo;
    }

    private void processGroupStateChange(String groupName, boolean selected) {
        java.util.List groupsList = new ArrayList();
        Object selectedGroup = primaryGroupBox.getSelectedItem();
        for (int i = primaryGroupBox.getItemCount() - 1; i >= 0; i--) {
            groupsList.add(primaryGroupBox.getItemAt(i));
        }
        if (selected) {
            groupsList.add(groupName);
        } else {
            groupsList.remove(groupName);
        }
        Collections.sort(groupsList);
        primaryGroupBox.removeAllItems();
        for (int i = 0, size = groupsList.size(); i < size; i++) {
            primaryGroupBox.addItem(groupsList.get(i));
        }

        if (selectedGroup == null) {
            selectedGroup = groupsList.get(0);
        }
        primaryGroupBox.setSelectedItem(selectedGroup);
    }

    private void tuneControls(JComponent chief) {
        boolean selected;
        if ((chief == null) || (accountExpBox == chief)) {
            selected = accountExpBox.isSelected();
            jLabelAccountExpired.setEnabled(isRoot() && selected);
            yearExpField.setEditable(selected);
            monthExpField.setEditable(selected);
            dayExpField.setEditable(selected);
        }

        if ((chief == null) || passExpBox == chief) {
            selected = passExpBox.isSelected();
            jLabelDaysBeforeAllowed.setEnabled(isRoot() && selected);
            jLabelDaysBeforeRequired.setEnabled(isRoot() && selected);
            jLabelDaysWarnBeforeChange.setEnabled(isRoot() && selected);
            jLabelDaysBeforeInactive.setEnabled(isRoot() && selected);
            daysBeforeAllowedField.setEditable(isRoot() && selected);
            daysBeforeRequiredField.setEditable(isRoot() && selected);
            daysWarnBeforeChField.setEditable(isRoot() && selected);
            daysBeforeAccInactiveField.setEditable(isRoot() && selected);
        }
    }


    private JPanel createUserDataTab() {
        JPanel panel = new JPanel(new GridBagLayout(), true);

        JLabel unLabel = new JLabel("User Name:");
        unLabel.setEnabled(isRoot());
        unLabel.setForeground(Color.black);
        constrain(panel, unLabel, 0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        userNameField = new JTextField();
        userNameField.setForeground(Color.black);
        userNameField.setEditable(isRoot());
        constrain(panel, userNameField, 1, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel fnLabel = new JLabel("Full Name:");
        fnLabel.setEnabled(isRoot());
        fnLabel.setForeground(Color.black);
        constrain(panel, fnLabel, 0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        fullNameField = new JTextField();
        fullNameField.setForeground(Color.black);
        fullNameField.setEditable(isRoot());
        constrain(panel, fullNameField, 1, 1, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel pLabel = new JLabel("Password:");
        pLabel.setEnabled(isRoot());
        pLabel.setForeground(Color.black);
        constrain(panel, pLabel, 0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        passwordField = new JPasswordField();
        passwordField.setForeground(Color.black);
        passwordField.setEditable(isRoot());
        constrain(panel, passwordField, 1, 2, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel cpLabel = new JLabel("Confirm Password:");
        cpLabel.setEnabled(isRoot());
        cpLabel.setForeground(Color.black);
        constrain(panel, cpLabel, 0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        confPasswordField = new JPasswordField();
        confPasswordField.setForeground(Color.black);
        confPasswordField.setEditable(isRoot());
        constrain(panel, confPasswordField, 1, 3, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel hdLabel = new JLabel("Home Directory:");
        hdLabel.setEnabled(isRoot());
        hdLabel.setForeground(Color.black);
        constrain(panel, hdLabel, 0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        homeDirField = new JTextField();
        homeDirField.setForeground(Color.black);
        homeDirField.setEditable(isRoot());
        constrain(panel, homeDirField, 1, 4, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel lsLabel = new JLabel("Login Shell:");
        lsLabel.setEnabled(isRoot());
        lsLabel.setForeground(Color.black);
        constrain(panel, lsLabel, 0, 5, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        loginShellField = new JTextField();
        loginShellField.setForeground(Color.black);
        loginShellField.setEditable(isRoot());
        constrain(panel, loginShellField, 1, 5, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel fillLabel = new JLabel();
        constrain(panel, fillLabel, 0, 6, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);

        return panel;
    }

    private JPanel createAccountInfoTab() {
        JPanel panel = new JPanel(new GridBagLayout(), true);

        accountExpBox = new JCheckBox("Enable account expiration");
        accountExpBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tuneControls(accountExpBox);
            }
        });
        accountExpBox.setEnabled(isRoot());
        constrain(panel, accountExpBox,
                0, 0, 2, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0,
                5, 5, 2, 5);

        constrain(panel, new JLabel(),
                2, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0,
                5, 5, 2, 5);

        jLabelAccountExpired = new JLabel("Account expires (YYYY-MM-DD):");
        jLabelAccountExpired.setForeground(Color.black);

        yearExpField = new JTextField();
        yearExpField.setEditable(isRoot());
        yearExpField.setPreferredSize(new Dimension(40, yearExpField.getPreferredSize().height));
        monthExpField = new JTextField();
        monthExpField.setPreferredSize(new Dimension(25, monthExpField.getPreferredSize().height));
        monthExpField.setEditable(isRoot());
        dayExpField = new JTextField();
        dayExpField.setPreferredSize(new Dimension(25, dayExpField.getPreferredSize().height));
        dayExpField.setEditable(isRoot());
        JPanel jPanelExp = new JPanel(new FlowLayout(FlowLayout.LEFT));
        jPanelExp.add(jLabelAccountExpired);
        jPanelExp.add(yearExpField);
        jPanelExp.add(new JLabel("-"));
        jPanelExp.add(monthExpField);
        jPanelExp.add(new JLabel("-"));
        jPanelExp.add(dayExpField);
        constrain(panel, jPanelExp,
                1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0,
                3, 20, 5, 5);

        accountLockedBox = new JCheckBox("User account is locked");
        accountLockedBox.setEnabled(isRoot());
        constrain(panel, accountLockedBox, 0, 2, 2, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        JLabel fillLabel = new JLabel();
        constrain(panel, fillLabel, 0, 3, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);

        return panel;
    }

    private JPanel createPasswordInfoTab() {
        JPanel panel = new JPanel(new GridBagLayout(), true);

        JLabel accountExpLabel = new JLabel("User last changed password on:");
        accountExpLabel.setEnabled(isRoot());
        accountExpLabel.setForeground(Color.black);
        constrain(panel, accountExpLabel, 0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        lastPassChangeLabel = new JLabel();
        lastPassChangeLabel.setEnabled(isRoot());
        lastPassChangeLabel.setForeground(Color.black);
        constrain(panel, lastPassChangeLabel, 1, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0, 5, 5, 5, 5);

        passExpBox = new JCheckBox("Enable password expiration");
        passExpBox.setEnabled(isRoot());
        passExpBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tuneControls(passExpBox);
            }
        });
        constrain(panel, passExpBox, 0, 1, 3, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0, 5, 5, 5, 5);

        jLabelDaysBeforeAllowed = new JLabel("Days before change allowed:");
        jLabelDaysBeforeAllowed.setForeground(Color.black);
        constrain(panel, jLabelDaysBeforeAllowed, 0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 25, 5, 5);

        daysBeforeAllowedField = new JTextField();
        daysBeforeAllowedField.setForeground(Color.black);
        daysBeforeAllowedField.setEditable(isRoot());
        constrain(panel, daysBeforeAllowedField, 1, 2, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        jLabelDaysBeforeRequired = new JLabel("Days before change required:");
        jLabelDaysBeforeRequired.setForeground(Color.black);
        constrain(panel, jLabelDaysBeforeRequired, 0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 25, 5, 5);

        daysBeforeRequiredField = new JTextField();
        daysBeforeRequiredField.setForeground(Color.black);
        daysBeforeRequiredField.setEditable(isRoot());
        constrain(panel, daysBeforeRequiredField, 1, 3, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        jLabelDaysWarnBeforeChange = new JLabel("Days warning before change:");
        jLabelDaysWarnBeforeChange.setForeground(Color.black);
        constrain(panel, jLabelDaysWarnBeforeChange, 0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 25, 5, 5);

        daysWarnBeforeChField = new JTextField();
        daysWarnBeforeChField.setForeground(Color.black);
        daysWarnBeforeChField.setEditable(isRoot());
        constrain(panel, daysWarnBeforeChField, 1, 4, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        jLabelDaysBeforeInactive = new JLabel("Days before account inactive:");
        jLabelDaysBeforeInactive.setForeground(Color.black);
        constrain(panel, jLabelDaysBeforeInactive, 0, 5, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 25, 5, 5);

        daysBeforeAccInactiveField = new JTextField();
        daysBeforeAccInactiveField.setForeground(Color.black);
        daysBeforeAccInactiveField.setEditable(isRoot());
        constrain(panel, daysBeforeAccInactiveField, 1, 5, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

//    JLabel fillLabel = new JLabel();
//    constrain(panel, fillLabel, 0, 6, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);

        return panel;
    }

    private JPanel createGroupsTab() {
        JPanel panel = new JPanel(new GridBagLayout(), true);

        JLabel iLabel = new JLabel("Select the groups that the user will be a member of:");
        iLabel.setEnabled(isRoot());
        iLabel.setForeground(Color.black);
        constrain(panel, iLabel, 0, 0, 2, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0, 5, 5, 5, 5);

        Object[] objects = new Object[10];
        for (int i = 0; i < objects.length; i++) {
            objects[i] = new JCheckBox("Group #" + i);
        }
        groupsSelectionList = new JList(objects) {
            private Object prevSelected;

            protected void processKeyEvent(KeyEvent e) {
                super.processKeyEvent(e);
                JCheckBox checkBox = (JCheckBox) getSelectedValue();
                if ((checkBox != null) && (e.getKeyCode() == KeyEvent.VK_SPACE) && (e.getID() == KeyEvent.KEY_PRESSED)) {
                    checkBox.setSelected(!checkBox.isSelected());
                    repaint();
                }
            }

            protected void processMouseEvent(MouseEvent e) {
                int id = e.getID();
                switch (id) {
                    case MouseEvent.MOUSE_PRESSED:
                        prevSelected = getSelectedValue();
                        super.processMouseEvent(e);
                        break;
                    case MouseEvent.MOUSE_CLICKED:
                        super.processMouseEvent(e);
                        if ((prevSelected != null) && (prevSelected == getSelectedValue()) && (MouseEvent.MOUSE_CLICKED == e.getID())) {
                            Rectangle bounds = getCellBounds(getSelectedIndex(), getSelectedIndex());
                            if ((e.getY() >= bounds.y) && (e.getY() <= bounds.y + bounds.height)) {
                                JCheckBox checkBox = (JCheckBox) getSelectedValue();
                                checkBox.setSelected(!checkBox.isSelected());
                                repaint();
                            }
                        }
                        break;
                    default:
                        super.processMouseEvent(e);
                        break;
                }


            }
        };
        groupsSelectionList.setCellRenderer(new ListCellRenderer() {
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                JComponent component = (JComponent) value;
                component.setBackground((isSelected) ? list.getSelectionBackground() : list.getBackground());
                if (isRoot()) {
                    component.setEnabled(true);
                } else {
                    component.setEnabled(false);
                }
                return component;
            }
        });
        groupsSelectionList.setEnabled(isRoot());
        constrain(panel, new JScrollPane(groupsSelectionList), 0, 1, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1.0, 1.0, 5, 5, 5, 5);

        JLabel pgLabel = new JLabel("Primary Group:");
        pgLabel.setForeground(Color.black);
        pgLabel.setEnabled(isRoot());
        constrain(panel, pgLabel, 0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        primaryGroupBox = new JComboBox(objects);
        primaryGroupBox.setEnabled(isRoot());
        constrain(panel, primaryGroupBox, 1, 2, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        return panel;
    }

    private boolean isRoot() {
        return root;
    }

    private static void constrain(Container container, Component component, int x, int y, int width, int height, int fill, int anchor, double weightx, double weighty, int insetTop, int insetLeft, int insetBottom, int insetRight) {
        Insets insets = new Insets(insetTop, insetLeft, insetBottom, insetRight);
        GridBagConstraints gbc = new GridBagConstraints(x, y, width, height, weightx, weighty, anchor, fill, insets, 0, 0);
        LayoutManager lm = container.getLayout();
        GridBagLayout gbl = (GridBagLayout) lm;
        gbl.setConstraints(component, gbc);
        container.add(component);
    }
}
