/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.tools;

import com.smardec.demo.j2native.Demo;
import com.smardec.j2native.*;

import java.io.IOException;
import java.net.ServerSocket;

//TODO: Add dispose calls to all callbacks

public class NativeLibraryHelper {
    //win
    public static final String WIN_KERNEL_NAME = "kernel32";
    public static final String WIN_USER_NAME = "user32";
    public static final String WIN_GDI_NAME = "gdi32";

    private static Library WIN_KERNEL;
    private static Library WIN_USER;

    //unix
    public static final String xNIX_KERNEL_NAME = "libc.so";
    public static final String SUNOS_KERNEL_NAME = xNIX_KERNEL_NAME;
    public static final String UNIX_KERNEL_NAME = "libc.so.6";
    public static final String UNIX_KERNEL_NAME_SECONDARY = "libc-2.3.2.so";

    private static int PORT = 13 * 666 + 7;

    private static NativeLibraryHelper instance;

    private NativeLibraryHelper() {
        init();
    }

    private void init() {
        if (SysInfoUtil.isUnix() || SysInfoUtil.isSunOS())
            System.setProperty(Library.LIBRARY_ENFORCE_KEY, Boolean.FALSE.toString());
        Library.ensureNativeCodeLoaded();
        if (SysInfoUtil.isWin())
            winInitLibManager();
        else if (SysInfoUtil.isUnix())
            unixInitLibManager();
        else if (SysInfoUtil.isSunOS())
            sunOSInitLibManager();
    }

    /**
     * Creates instance of NativeLibraryHelper. Called ensureNativeCodeLoaded and  xxxInitLibManager.
     *
     * @return NativeLibraryHelper instance
     */
    public static NativeLibraryHelper getInstance() {
        if (instance == null) {
            instance = new NativeLibraryHelper();
        }
        return instance;
    }

    /**
     * Indicates whether demo is alredy started.
     *
     * @return <code>false</code> is already started
     */
    public boolean ensureOneInstance() {
        if (SysInfoUtil.isWin())
            return ensureOneInstanceWin();
        else if (SysInfoUtil.isUnix() || SysInfoUtil.isSunOS())
            return ensureOneInstanceUnix();
        return false;
    }

    private boolean ensureOneInstanceWin() {
        Pointer mutexName = new Pointer(new AnsiString("com.smardec.demo.j2native.tools.NativeLibraryHelper"));
        UInt32 desiredAcces = new UInt32(0x1F0001);
        Bool inheritHandle = new Bool(false);
        Pointer.Void mutexHandle = new Pointer.Void();
        Function openMutex = WIN_KERNEL.getFunction("OpenMutexA");
        openMutex.invoke(mutexHandle, desiredAcces, inheritHandle, mutexName);
        if (!mutexHandle.isNull()) {
            // Mutex exists - one instance is already running
            return false;
        }

        // Not yet running - lock by creating mutex
        Pointer.Void mutexAttributes = new Pointer.Void(0);
        Bool initialOwner = new Bool(false);
        Function createMutex = WIN_KERNEL.getFunction("CreateMutexA");
        createMutex.invoke(mutexHandle, mutexAttributes, initialOwner, mutexName);
        if (mutexHandle.isNull()) {
            throw new RuntimeException("Mutex creation failed, last error = " + getLastErrorWin(true));
        }
        return true;
    }

    private boolean ensureOneInstanceUnix() {
        try {
            new ServerSocket(PORT);
        } catch (java.net.BindException e) {
            return false;
        } catch (final IOException e) { // an unexpected exception occurred
            return true;
        }
        return true;
    }

    ///////////////////////////WIN FUNCTIONS////////////////////////////////////
    public void messageBeepWin(int tone) {
        Function messageBeep = WIN_USER.getFunction("MessageBeep");
        messageBeep.invoke(null, new Int32(tone));
    }

    public void showMessageBoxWin(String caption, String message, int flags) {
        Pointer.Void hWnd = new Pointer.Void();
        Pointer pText = new Pointer(new AnsiString(message));
        Pointer pCaption = new Pointer(new AnsiString(caption));
        UInt uFlags = new UInt(flags);
        // For functions taking strings as parameters there are two versions: ANSI and Unicode
        // We chose ANSI here and hence function name ends with A
        // It should be noted that there is no plain MessageBox function exported
        Function findWindow = WIN_USER.getFunction("FindWindowA");
        findWindow.invoke(hWnd, new Pointer.Void(), new Pointer(new AnsiString(Demo.WINDOW_CAPTION)));

        Function function = WIN_USER.getFunction("MessageBoxA");
        function.invoke(null, hWnd, pText, pCaption, uFlags);
    }

    private long getLastErrorWin(boolean clear) {
        UInt32 r = new UInt32();
        Function getLE = WIN_KERNEL.getFunction("GetLastError");
        getLE.invoke(r);
        long errCode = r.longValue();
        if (clear) {
            UInt32 zero = new UInt32(0);
            Function setLE = WIN_KERNEL.getFunction("SetLastError");
            setLE.invoke(null, zero);
        }
        return errCode;
    }

    ///////////////////////////UNIX FUNCTIONS///////////////////////////////////

    /**
     * Resets lib manager.
     */
    public void reset() {
        LibraryManager.getInstance().reset();
    }

    private void winInitLibManager() {
        LibraryManager libManager = LibraryManager.getInstance();
        Library library = new Library(WIN_KERNEL_NAME);
        libManager.registerLibrary(library);
        library = new Library(WIN_USER_NAME);
        libManager.registerLibrary(library);
        library = new Library(WIN_GDI_NAME);
        libManager.registerLibrary(library);

        WIN_KERNEL = libManager.getLibrary(WIN_KERNEL_NAME);
        WIN_USER = libManager.getLibrary(WIN_USER_NAME);
    }

    private void unixInitLibManager() {
        LibraryManager libManager = LibraryManager.getInstance();
        Library library = new Library(UNIX_KERNEL_NAME);
        try {
            libManager.registerLibrary(xNIX_KERNEL_NAME, library);
        } catch (LibraryNotFoundException e) {
            library = new Library(UNIX_KERNEL_NAME_SECONDARY);
            libManager.registerLibrary(xNIX_KERNEL_NAME, library);
        }
    }

    private void sunOSInitLibManager() {
        LibraryManager libManager = LibraryManager.getInstance();
        Library library = new Library(SUNOS_KERNEL_NAME);
        libManager.registerLibrary(xNIX_KERNEL_NAME, library);
    }
}
